@@ -16,20 +16,11 @@ import (
 	"strings"
 	"time"
 
-	"github.com/openshift/hive/pkg/gcpclient"
-
 	"github.com/pkg/errors"
 	log "github.com/sirupsen/logrus"
 	"github.com/spf13/cobra"
 	"sigs.k8s.io/yaml"
 
-	contributils "github.com/openshift/hive/contrib/pkg/utils"
-	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
-	"github.com/openshift/hive/pkg/constants"
-	controllerutils "github.com/openshift/hive/pkg/controller/utils"
-	"github.com/openshift/hive/pkg/resource"
-
-	k8slabels "github.com/openshift/hive/pkg/util/labels"
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/meta"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -39,28 +30,35 @@ import (
 	utilrand "k8s.io/apimachinery/pkg/util/rand"
 	"k8s.io/apimachinery/pkg/util/wait"
 	"k8s.io/apimachinery/pkg/watch"
+	"k8s.io/client-go/kubernetes/scheme"
 	"k8s.io/client-go/rest"
 	"k8s.io/client-go/tools/cache"
 	clientwatch "k8s.io/client-go/tools/watch"
-	"k8s.io/utils/pointer"
-
-	"k8s.io/client-go/kubernetes/scheme"
 	"k8s.io/client-go/util/retry"
-
+	"k8s.io/utils/pointer"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/client/apiutil"
-	"sigs.k8s.io/controller-runtime/pkg/controller/controllerutil"
 
-	azuresession "github.com/openshift/installer/pkg/asset/installconfig/azure"
 	"github.com/openshift/installer/pkg/destroy/aws"
 	"github.com/openshift/installer/pkg/destroy/azure"
 	"github.com/openshift/installer/pkg/destroy/gcp"
 	"github.com/openshift/installer/pkg/destroy/openstack"
+	"github.com/openshift/installer/pkg/destroy/ovirt"
+	"github.com/openshift/installer/pkg/destroy/providers"
 	"github.com/openshift/installer/pkg/destroy/vsphere"
 	installertypes "github.com/openshift/installer/pkg/types"
+	installertypesazure "github.com/openshift/installer/pkg/types/azure"
 	installertypesgcp "github.com/openshift/installer/pkg/types/gcp"
 	installertypesopenstack "github.com/openshift/installer/pkg/types/openstack"
+	installertypesovirt "github.com/openshift/installer/pkg/types/ovirt"
 	installertypesvsphere "github.com/openshift/installer/pkg/types/vsphere"
+
+	contributils "github.com/openshift/hive/contrib/pkg/utils"
+	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
+	"github.com/openshift/hive/pkg/constants"
+	"github.com/openshift/hive/pkg/gcpclient"
+	"github.com/openshift/hive/pkg/resource"
+	k8slabels "github.com/openshift/hive/pkg/util/labels"
 )
 
 const (
@@ -529,58 +527,32 @@ func (m *InstallManager) cleanupFailedInstall(cd *hivev1.ClusterDeployment, prov
 }
 
 func cleanupFailedProvision(dynClient client.Client, cd *hivev1.ClusterDeployment, infraID string, logger log.FieldLogger) error {
+	var uninstaller providers.Destroyer
 	switch {
 	case cd.Spec.Platform.AWS != nil:
 		// run the uninstaller to clean up any cloud resources previously created
 		filters := []aws.Filter{
 			{kubernetesKeyPrefix + infraID: "owned"},
 		}
-		uninstaller := &aws.ClusterUninstaller{
+		uninstaller = &aws.ClusterUninstaller{
 			Filters: filters,
 			Region:  cd.Spec.Platform.AWS.Region,
 			Logger:  logger,
 		}
-
-		if err := uninstaller.Run(); err != nil {
-			return err
-		}
-
-		// If we're managing DNS for this cluster, lookup the DNSZone and cleanup
-		// any leftover A records that may have leaked due to
-		// https://jira.coreos.com/browse/CORS-1195.
-		if cd.Spec.ManageDNS {
-			dnsZone := &hivev1.DNSZone{}
-			dnsZoneNamespacedName := types.NamespacedName{Namespace: cd.Namespace, Name: controllerutils.DNSZoneName(cd.Name)}
-			err := dynClient.Get(context.TODO(), dnsZoneNamespacedName, dnsZone)
-			if err != nil {
-				logger.WithError(err).Error("error looking up managed dnszone")
-				return err
-			}
-			if dnsZone.Status.AWS == nil {
-				return fmt.Errorf("found non-AWS DNSZone for AWS ClusterDeployment")
-			}
-			if dnsZone.Status.AWS.ZoneID == nil {
-				// Shouldn't really be possible as we block install until DNS is ready:
-				return fmt.Errorf("DNSZone %s has no ZoneID set", dnsZone.Name)
-			}
-			return cleanupDNSZone(*dnsZone.Status.AWS.ZoneID, cd.Spec.Platform.AWS.Region, logger)
-		}
-		return nil
 	case cd.Spec.Platform.Azure != nil:
-		uninstaller := &azure.ClusterUninstaller{}
-		uninstaller.Logger = logger
-		session, err := azuresession.GetSession()
+		metadata := &installertypes.ClusterMetadata{
+			InfraID: infraID,
+			ClusterPlatformMetadata: installertypes.ClusterPlatformMetadata{
+				Azure: &installertypesazure.Metadata{
+					CloudName: installertypesazure.PublicCloud,
+				},
+			},
+		}
+		var err error
+		uninstaller, err = azure.New(logger, metadata)
 		if err != nil {
 			return err
 		}
-
-		uninstaller.InfraID = infraID
-		uninstaller.SubscriptionID = session.Credentials.SubscriptionID
-		uninstaller.TenantID = session.Credentials.TenantID
-		uninstaller.GraphAuthorizer = session.GraphAuthorizer
-		uninstaller.Authorizer = session.Authorizer
-
-		return uninstaller.Run()
 	case cd.Spec.Platform.GCP != nil:
 		credsFile := os.Getenv("GOOGLE_CREDENTIALS")
 		projectID, err := gcpclient.ProjectIDFromFile(credsFile)
@@ -596,11 +568,10 @@ func cleanupFailedProvision(dynClient client.Client, cd *hivev1.ClusterDeploymen
 				},
 			},
 		}
-		uninstaller, err := gcp.New(logger, metadata)
+		uninstaller, err = gcp.New(logger, metadata)
 		if err != nil {
 			return err
 		}
-		return uninstaller.Run()
 	case cd.Spec.Platform.OpenStack != nil:
 		metadata := &installertypes.ClusterMetadata{
 			InfraID: infraID,
@@ -613,11 +584,11 @@ func cleanupFailedProvision(dynClient client.Client, cd *hivev1.ClusterDeploymen
 				},
 			},
 		}
-		uninstaller, err := openstack.New(logger, metadata)
+		var err error
+		uninstaller, err = openstack.New(logger, metadata)
 		if err != nil {
 			return err
 		}
-		return uninstaller.Run()
 	case cd.Spec.Platform.VSphere != nil:
 		vSphereUsername := os.Getenv(constants.VSphereUsernameEnvVar)
 		if vSphereUsername == "" {
@@ -637,15 +608,35 @@ func cleanupFailedProvision(dynClient client.Client, cd *hivev1.ClusterDeploymen
 				},
 			},
 		}
-		uninstaller, err := vsphere.New(logger, metadata)
+		var err error
+		uninstaller, err = vsphere.New(logger, metadata)
+		if err != nil {
+			return err
+		}
+	case cd.Spec.Platform.Ovirt != nil:
+		metadata := &installertypes.ClusterMetadata{
+			InfraID: infraID,
+			ClusterPlatformMetadata: installertypes.ClusterPlatformMetadata{
+				Ovirt: &installertypesovirt.Metadata{
+					ClusterID: cd.Spec.Platform.Ovirt.ClusterID,
+				},
+			},
+		}
+		var err error
+		uninstaller, err = ovirt.New(logger, metadata)
 		if err != nil {
 			return err
 		}
-		return uninstaller.Run()
 	default:
 		logger.Warn("unknown platform for re-try cleanup")
 		return errors.New("unknown platform for re-try cleanup")
 	}
+
+	if err := uninstaller.Run(); err != nil {
+		return err
+	}
+
+	return cleanupDNSZone(dynClient, cd, logger)
 }
 
 // generateAssets runs openshift-install commands to generate on-disk assets we need to
@@ -1082,11 +1073,21 @@ func uploadAdminKubeconfig(provision *hivev1.ClusterProvision, m *InstallManager
 	m.log.WithField("derivedObject", kubeconfigSecret.Name).Debug("Setting labels on derived object")
 	kubeconfigSecret.Labels = k8slabels.AddLabel(kubeconfigSecret.Labels, constants.ClusterProvisionNameLabel, provision.Name)
 	kubeconfigSecret.Labels = k8slabels.AddLabel(kubeconfigSecret.Labels, constants.SecretTypeLabel, constants.SecretTypeKubeConfig)
-	if err := controllerutil.SetControllerReference(provision, kubeconfigSecret, scheme.Scheme); err != nil {
-		m.log.WithError(err).Error("error setting controller reference on kubeconfig secret")
+
+	provisionGVK, err := apiutil.GVKForObject(provision, scheme.Scheme)
+	if err != nil {
+		m.log.WithError(err).Errorf("error getting GVK for provision")
 		return nil, err
 	}
 
+	kubeconfigSecret.OwnerReferences = []metav1.OwnerReference{{
+		APIVersion:         provisionGVK.GroupVersion().String(),
+		Kind:               provisionGVK.Kind,
+		Name:               provision.Name,
+		UID:                provision.UID,
+		BlockOwnerDeletion: pointer.BoolPtr(true),
+	}}
+
 	if err := createWithRetries(kubeconfigSecret, m); err != nil {
 		return nil, err
 	}
@@ -1126,11 +1127,21 @@ func uploadAdminPassword(provision *hivev1.ClusterProvision, m *InstallManager)
 	m.log.WithField("derivedObject", s.Name).Debug("Setting labels on derived object")
 	s.Labels = k8slabels.AddLabel(s.Labels, constants.ClusterProvisionNameLabel, provision.Name)
 	s.Labels = k8slabels.AddLabel(s.Labels, constants.SecretTypeLabel, constants.SecretTypeKubeAdminCreds)
-	if err := controllerutil.SetControllerReference(provision, s, scheme.Scheme); err != nil {
-		m.log.WithError(err).Error("error setting controller reference on kubeconfig secret")
+
+	provisionGVK, err := apiutil.GVKForObject(provision, scheme.Scheme)
+	if err != nil {
+		m.log.WithError(err).Errorf("error getting GVK for provision")
 		return nil, err
 	}
 
+	s.OwnerReferences = []metav1.OwnerReference{{
+		APIVersion:         provisionGVK.GroupVersion().String(),
+		Kind:               provisionGVK.Kind,
+		Name:               provision.Name,
+		UID:                provision.UID,
+		BlockOwnerDeletion: pointer.BoolPtr(true),
+	}}
+
 	if err := createWithRetries(s, m); err != nil {
 		return nil, err
 	}
@@ -1198,7 +1209,7 @@ func (m *InstallManager) cleanupAdminPasswordSecret() error {
 }
 
 // deleteAnyExistingObject will look for any object that exists that matches the passed in 'obj' and will delete it if it exists
-func (m *InstallManager) deleteAnyExistingObject(namespacedName types.NamespacedName, obj runtime.Object) error {
+func (m *InstallManager) deleteAnyExistingObject(namespacedName types.NamespacedName, obj hivev1.MetaRuntimeObject) error {
 	return resource.DeleteAnyExistingObject(m.DynamicClient, namespacedName, obj, m.log)
 }
 
